# Copyright 2006 Laboratoire de Biologie et de Biometrie Appliquée 
# (UMR 5558);CNRS; Univ. Lyon 1, 43 bd 11 nov, 69622, 
# Villeurbanne Cedex, France.
#
# This file is part of MareyBase.
#
# MMSpline3 is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# MMSpline3 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with MMSpline3; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


#--- MMLoess ---------------------------------------------------------------

#--- class definition

setClass("MMSpline3",
	contains="Interpolation",
# ADD INTERNAL DATA DEFINITION HERE (params etc...)
    representation(
        type = "character",
        gcv = "logical",
        df = "numeric",
        spar = "numeric",
        model = "ANY"
    ),
	prototype(
        type = "cross-validation",
        df = 10,
        spar = 0.1,
        gcv = FALSE,
        model = NULL
    )
)

#--- constructors

setGeneric("MMSpline3",
	function(val) standardGeneric("MMSpline3"))

setMethod("MMSpline3",	
	signature(val="missing"),
	function(val){
		object <- new("MMSpline3")
# FURTHER INITIALISATION CODE HERE
		object@visible <- T
		object@color <- "#000000"
		object
	}
)

#--- accessors

setGeneric("type", 
	function(object) standardGeneric("type"))

setMethod("type","MMSpline3",
	function(object)object@type)
 

setGeneric("spar", 
	function(object) standardGeneric("spar"))

setMethod("spar","MMSpline3",
	function(object){
        return(object@spar)
    }
)
    
setGeneric("df", 
	function(object) standardGeneric("df"))

setMethod("df","MMSpline3",
	function(object){
        return(object@df)
    }
)

setGeneric("gcv", 
	function(object) standardGeneric("gcv"))

setMethod("gcv","MMSpline3",
	function(object){
        return(object@gcv)
    }
)

#--- replacement methods

setGeneric("type<-",
	function(object,value) standardGeneric("type<-"))

setReplaceMethod("type",c("MMSpline3","character"),
	function(object,value){
		if(value=="cross-validation" ||
            value=="spar"||
            value=="degree of freedom")
            object@type <- value 
        object
    }
)

setGeneric("spar<-",
	function(object,value) standardGeneric("spar<-"))

setReplaceMethod("spar",c("MMSpline3","numeric"),
	function(object,value){
		if(value > 0)
			object@spar <- value
		object
	}
)

setGeneric("df<-",
	function(object,value) standardGeneric("df<-"))

setReplaceMethod("df",c("MMSpline3","numeric"),
	function(object,value){
		if(value > 0)
			object@df <- value
		object
	}
)

setGeneric("gcv<-",
	function(object,value) standardGeneric("gcv<-"))

setReplaceMethod("gcv",c("MMSpline3","logical"),
	function(object,value){
        object@gcv <- value
		object
	}
)

#--- query

setMethod("query",c("MMSpline3","numeric"),function(object,pos){
    sapply(pos, function(pp){
        posplus <- pp+1
        posmoins <- pp-1
        pred <- predict(object@model,c(posmoins,posplus))$y
        round((pred[[2]]-pred[[1]])/
            ((posplus-posmoins)/1000000),digits=2)
    })
})

#--- interpolate

setMethod("interpolate",
	signature(object="MMSpline3",map="MareyMap"),
	function(object,map){
# THE ACTUAL CALCULATION CODE GOES HERE
		valgen <- geneticDistances(map)[which(markerValidity(map))]
		valphys <- physicalPositions(map)[which(markerValidity(map))]
		object@physicalPositions <- valphys
        cl <- call("smooth.spline",x=valphys,y=valgen)
        if(object@type == "cross-validation"){
            if(object@gcv)
                cl$cv <- FALSE
            else
                cl$cv <- TRUE
        }else if(object@type == "spar"){
            cl$spar <- object@spar
        }else if(object@type == "degree of freedom"){
            cl$df <- object@df
        }
		object@model <- eval(cl)
        object@rates <- query(object,physicalPositions(map))
        object
	}
)

#--- plot

setMethod("plot",c("MareyMap","MMSpline3"),function(x,y,...){
	#valphys <- physicalPositions(x)[which(markerValidity(x))]
#	plpar <- list(...)
#	plpar$col <- y@color
#	
#	if(is.null(plpar[["xlim"]]))
#		plpar[["xlim"]] <- 
#				c(0,max(physicalPositions(x)))
#	if(is.null(plpar[["ylim"]]))
#		plpar[["ylim"]] <- 
#				c(0,max(geneticDistances(x)))
#	ord <- order(valphys)
#		plpar[["type"]] <- "l"
#		expli <- seq(from=min(valphys),
#							to=max(valphys),
#							length.out=1000)
#		plpar[["x"]] <- expli/1000000
#		plpar[["y"]] <- predict(y@model,newdata=data.frame(expli))
#		do.call("plot",plpar)
#		par(new=T)
#		plpar[["ylim"]] <- plpar[["ylimr"]]
#		#physical positions +1
#		pp1 <- expli+1
#		#physical positions -1
#		pm1 <- expli-1
#		gp1 <- predict(y@model,newdata=pp1)
#		#predicted genetical positions @ phys-1
#		gm1 <- predict(y@model,newdata=pm1)
#		plpar[["y"]] <- mapply(function(xa,xb,ya,yb){
#			(yb-ya)/((xb-xa)/1000000)
#		},pm1,pp1,gm1,gp1)
#		do.call("plot",plpar)
	plotModel(y,...)
	par(new=T)
	plotRates(y,...)
})

#--- plotModel

setMethod("plotModel","MMSpline3",function(object,...){
# Method called for plotting the interpolation model (if any)

	cl <- match.call()
	cl[[1]] <- as.name("plot")
	cl$col <- object@color
	cl$type = "l"
	expli <- seq(from=min(object@physicalPositions),
						to=max(object@physicalPositions),
						length.out=100)
	cl$x <- expli/1000000
	cl$y <- predict(object@model,expli)$y
	eval(cl)
})

#--- plotRate

setMethod("plotRate","MMSpline3",function(object,...){
# method called to plot the recombination rates

	cl <- match.call()
	cl[[1]] <- as.name("plot")
	cl$object <- NULL
	cl$col <- object@color
	cl$type <- "l"
	expli <- seq(from=min(object@physicalPositions),
						to=max(object@physicalPositions),
						length.out=100)
	cl$x <- expli/1000000
	#physical positions +1
#	pp1 <- expli+1
	#physical positions -1
#	pm1 <- expli-1
	#predicted genetical positions @ phys+1
#	gp1 <- predict(object@model,newdata=pp1)
	#predicted genetical positions @ phys-1
#	gm1 <- predict(object@model,newdata=pm1)
#	cl$y <- mapply(function(xa,xb,ya,yb){
#		(yb-ya)/((xb-xa)/1000000)
#	},pm1,pp1,gm1,gp1)
    cl$y <- query(object,expli)
	eval(cl)
})

#--- createOrder
setMethod("createOrder",
	"MMSpline3",
	function(object){
		paste("new(\"MMSpline3\",",argList(object),
                ",spar = ",object@spar,
				", df = ",object@df,")",sep="")
	}
)

#--- userParam

setMethod("userParam",
	"MMSpline3",
	function(object){	
        type <- InterpolationParam()
		paramName(type) <- "type"
		paramDesc(type) <- "\n\
There are three different ways of estimating the\n\
parameters of locally fitted polynomials.\n\
If you choose 'spar' here, you will have to set\n\
the value for the parameter 'spar' below. If you\n\
select 'degree of freedom', you will have to set\n\
the value of the parameter 'degree of freedom'.\n\
If you choose 'cross-validation', you have to\n\
choose whether or not to use 'generalised cross-\n\
validation'.\n\
Using 'cross-validation' with generalised cross-\n\
validation set off is generally a good choice on\n\
a clean map."
		paramType(type) <- "list"
        paramValues(type) <- c("cross-validation","spar","degree of freedom")
        paramDefault(type) <- "cross-validation"
		paramFun(type) <- "type"
		

		spar <- InterpolationParam()
		paramName(spar) <- "spar"
		paramDesc(spar) <- "\n\
Spar is the smoothing parameter, similar to span\n\
in loess. The higher it is, the smoother the curve\n\
gets."
		paramType(spar) <- "numeric"
		paramDefault(spar) <- 0.1
		paramMin(spar) <- 0.0000001
		paramMax(spar) <- 1
		paramFun(spar) <- "spar"
		
		df <- InterpolationParam()
		paramName(df) <- "degree of freedom"
		paramDesc(df) <- "\n\
Controls the amount of smoothing by setting the\n\
degree of freedom, which corresponds to the trace\n\
of the smoothing matrix. It is not intuitively\n\
obvious how to choose a value for this parameter\n\
and is often more convinient to rely on spar or\n\
cross-validation."
		paramType(df) <- "integer"
		paramDefault(df) <- 2
		paramMin(df) <- 0
		paramMax(df) <- 100	
		paramFun(df) <- "df"
	
        gcv<- InterpolationParam()
		paramName(gcv) <- "generalised cross-validation"
		paramDesc(gcv) <- "\n\
wether or not Generalised Cross Validation shall\n\
be used instead of usual cross validation. The\n\
'generalized' cross-validation method should be\n\
used when there are duplicated points in ’x’."
		paramType(gcv) <- "logical"
		paramDefault(gcv) <- FALSE
		paramFun(gcv) <- "gcv"
    
		c(userParam(as(object,"Interpolation")),list(type,spar,df,gcv))
	}
)

registerInterpolationMethod("Cubic splines","MMSpline3")
#--- end -----------------------------------------------------------------------
