\name{Interpolation-class}
\docType{class}
\alias{Interpolation}
\alias{Interpolation-class}
\alias{argList,Interpolation-method}
\alias{color<-,Interpolation-method}
\alias{color,Interpolation-method}
\alias{interpolate,Interpolation,MareyMap-method}
\alias{name<-,Interpolation-method}
\alias{name,Interpolation-method}
\alias{persistent<-,Interpolation-method}
\alias{persistent,Interpolation-method}
\alias{plot,MareyMap,Interpolation-method}
\alias{plotModel,Interpolation-method}
\alias{plotRate,Interpolation-method}
\alias{query,Interpolation,integer-method}
\alias{rates<-,Interpolation-method}
\alias{rates,Interpolation-method}
\alias{userParam,Interpolation-method}
\alias{visible<-,Interpolation-method}
\alias{visible,Interpolation-method}

\title{Class "Interpolation" }
\description{	This class serves as a base for interpolation methods. Although it is not defined as virtual for technical reason, it should not be used as is and is only useful after having been extended. }
\section{Objects from the Class}{
Objects from the class should not be created
}
\section{Slots}{
	 \describe{
    \item{\code{name}:}{Object of class \code{"character"} holding the name of the interpolation }
    \item{\code{color}:}{Object of class \code{"character"} holding the name or the code (\#RRGGBB) of the color of the interpolation on the plot }
    \item{\code{physicalPositions}:}{Object of class \code{"vector"} holding the valid physical position of the map on which the interpolation is calculated. This redundancy is necessary to cary out the plotting of the map. }
    \item{\code{rates}:}{Object of class \code{"vector"} holding the local recombination rate estimations at the location of each markers. }
    \item{\code{visible}:}{Object of class \code{"logical"}, wether or not the interpolation should be plotted }
    \item{\code{persistent}:}{Object of class \code{"logical"}, wether or not the interpolation should be saved when the map is saved to text file. }
  }
}
\section{Methods}{
  \describe{
    \item{argList}{\code{signature(object = "Interpolation")}: This function is used in the function createList (c.f. code examples). }
    \item{color<-}{\code{signature(object = "Interpolation")}: changes the color of the interpolation. }
    \item{color}{\code{signature(object = "Interpolation")}: returns the color of the interpolation. }
    \item{interpolate}{\code{signature(object = "Interpolation", map = "MareyMap")}: recalculate the interpolation. }
    \item{name<-}{\code{signature(object = "Interpolation")}: changes the name of the interpolation. }
    \item{name}{\code{signature(object = "Interpolation")}: returns the name of the interpolation. }
    \item{persistent<-}{\code{signature(object = "Interpolation")}: changes the persistency state of the interpolation.  }
    \item{persistent}{\code{signature(object = "Interpolation")}: returns the persistency state of the interpolation. }
    \item{plotModel}{\code{signature(object = "Interpolation")}: this function plots only the model of the marey map calculated by the method. }
    \item{plotRate}{\code{signature(object = "Interpolation")}: this function plots the recombination rates calculated by the method. }
    \item{query}{\code{signature(object = "Interpolation", pos = "integer")}: returns the recombinatioin rate calculated by this method on a given location on the sequence. }
    \item{rates<-}{\code{signature(object = "Interpolation")}: replaces the rates with a new vector. }
    \item{rates}{\code{signature(object = "Interpolation")}: returns the rates of the interpolation. }
    \item{userParam}{\code{signature(object = "Interpolation")}: function returning information about the parameters modifiable by the user (c.f. code example.) }
    \item{visible<-}{\code{signature(object = "Interpolation")}: changes the visibility of the interpolation. }
    \item{visible}{\code{signature(object = "Interpolation")}: returns the visibility of the interpolation }
	 }
}
\references{ ~put references to the literature/web site here ~ }
\author{ Clement Rezvoy <rezvoy@biomserv.univ-lyon1.fr> }

\seealso{
	\code{\link{InterpolationParam-class}}
	\code{\link{MareyMap-class}}
}
\examples{

###
# Example of extension of the class Interpolation
###

	#--- class definition
	
	setClass("MMLoess",
# The new class extends Interpolation
		contains="Interpolation",
# and then defines its own attributes
		representation(span = "numeric",
							degree = "integer",
							model = "ANY"
							),
		prototype(	span=0.1,
						degree=as.integer(2),
						model = NULL
						)
	)


	#--- constructors

	setGeneric("MMLoess",
		function(val) standardGeneric("MMLoess"))
	
	setMethod("MMLoess",	
		signature(val="missing"),
		function(val){
			object <- new("MMLoess")
			object@visible <- TRUE
			object@color <- "#000000"
			object
		}
	)

	#--- accessors

	setGeneric("span", 
		function(object) standardGeneric("span"))
	
	setMethod("span","MMLoess",
		function(object)object@span)

	setGeneric("degree", 
		function(object) standardGeneric("degree"))
	
	setMethod("degree","MMLoess",
		function(object)object@degree)

	#--- replacement methods

	setGeneric("span<-",
		function(object,value) standardGeneric("span<-"))

	setReplaceMethod("span","MMLoess",
		function(object,value){
			if(value > 0)
				object@span <- value
			object
		}
	)

	setGeneric("degree<-",
		function(object,value) standardGeneric("degree<-"))

	setReplaceMethod("degree","MMLoess",
		function(object,value){
			if(value > 0 && value < 3)
				object@degree <- as.integer(value)
			object
		}
	)
	
# Here are the functions that must be redefined:

# query is the function that returns the recombination rate on a specific 
# physical point
	setMethod("query",c("MMLoess","numeric"),function(object,pos){
		posplus <- pos+1
		posmoins <- pos-1
		pred <- predict(object@model,newdata=c(posmoins,posplus))		
		round((pred[[2]]-pred[[1]])/((posplus-posmoins)/1000000),digits=2)
	})

# Interpolate is the function that carries out the calculation of the 
# interpolation. It must at least populate the vector rates with the 
# recombination rates at the position of each markers but it may also save 
# more detailed results elsewhere in the class.
	setMethod("interpolate",
		signature(object="MMLoess",map="MareyMap"),
		function(object,map){
			valgen <- geneticDistances(map)[which(markerValidity(map))]
			valphys <- physicalPositions(map)[which(markerValidity(map))]
			object@physicalPositions <- valphys
			object@model <- loess(	valgen ~ valphys, 
											span=object@span,
											degree=object@degree)
			#physical positions +1
			pp1 <- physicalPositions(map)+1
			#physical positions -1
			pm1 <- physicalPositions(map)-1
			#predicted genetical positions @ phys+1
			gp1 <- predict(object@model,newdata=pp1)
			#predicted genetical positions @ phys-1
			gm1 <- predict(object@model,newdata=pm1)
			object@rates <- mapply(function(xa,xb,ya,yb){
				round((yb-ya)/((xb-xa)/1000000),2)
			},pm1,pp1,gm1,gp1)
			object
		}
	)

# PlotModel is the function called to plot the interpolation's model of the 
# marey map. If this function is not redefined, nothing will be plotted 
	#--- plotModel
	setMethod("plotModel","MMLoess",function(object,...){
		cl <- match.call()
		cl[[1]] <- as.name("plot")
		cl$col <- object@color
		cl$type = "l"
		expli <- seq(from=min(object@physicalPositions),
							to=max(object@physicalPositions),
							length.out=1000)
		cl$x <- expli/1000000
		cl$y <- predict(object@model,newdata=data.frame(expli))
		eval(cl)
	})
	
# PlotModel is the function called to plot the recombination rates estimated 
# with this method for the current map. 	
	setMethod("plotRate","MMLoess",function(object,...){
		cl <- match.call()
		cl[[1]] <- as.name("plot")
		cl$object <- NULL
		cl$col <- object@color
		cl$type <- "l"
		expli <- seq(from=min(object@physicalPositions),
							to=max(object@physicalPositions),
							length.out=1000)
		cl$x <- expli/1000000
		#physical positions +1
		pp1 <- expli+1
		#physical positions -1
		pm1 <- expli-1
		#predicted genetical positions @ phys+1
		gp1 <- predict(object@model,newdata=pp1)
		#predicted genetical positions @ phys-1
		gm1 <- predict(object@model,newdata=pm1)
		cl$y <- mapply(function(xa,xb,ya,yb){
			(yb-ya)/((xb-xa)/1000000)
		},pm1,pp1,gm1,gp1)
		eval(cl)
	})
	
# createOrder is used when a map is saved to text file with its interpolations. 
# This function returns the order to recreate the interpolation when the file 
# read in. This will be saved at the top of the text file as a comment. 
	setMethod("createOrder",
		"MMLoess",
		function(object){
			paste("new(\"MMLoess\",",argList(object),", span = ",object@span,
					", degree = as.integer(",object@degree,"))",sep="")
		}
	)
	
# userParam returns information about the parameters accessible to the user 
# as a vector of objects of type interpolationParam. 
# These informations can be used by the GUI to dynamically create the dialog 
# to modify these parameters.
	setMethod("userParam",
		"MMLoess",
		function(object){	
			span <- InterpolationParam()
			paramName(span) <- "span"
			paramDesc(span) <- "span\n"
			paramType(span) <- "numeric"
			paramDefault(span) <- 0.1
			paramMin(span) <- 0
			paramMax(span) <- 1	
			paramFun(span) <- "span"
			
			deg <- InterpolationParam()
			paramName(deg) <- "degree"
			paramDesc(deg) <- "degree of the fitted polynomial"
			paramType(deg) <- "integer"
			paramDefault(deg) <- 2
			paramMin(deg) <- 0
			paramMax(deg) <- 2	
			paramFun(deg) <- "degree"
			# basic user params are defined in the class Interpolation.  
			c(userParam(as(object,"Interpolation")),list(span,deg))
		}
	)

# this last line is necessary to register the method so that it is 
# accessible via the GUI
	registerInterpolationMethod("loess based method","MMLoess")
}
\keyword{classes}
