### Name: Interpolation-class
### Title: Class "Interpolation"
### Aliases: Interpolation Interpolation-class argList,Interpolation-method
###   color<-,Interpolation-method color,Interpolation-method
###   interpolate,Interpolation,MareyMap-method name<-,Interpolation-method
###   name,Interpolation-method persistent<-,Interpolation-method
###   persistent,Interpolation-method plot,MareyMap,Interpolation-method
###   plotModel,Interpolation-method plotRate,Interpolation-method
###   query,Interpolation,integer-method rates<-,Interpolation-method
###   rates,Interpolation-method userParam,Interpolation-method
###   visible<-,Interpolation-method visible,Interpolation-method
### Keywords: classes

### ** Examples


###
# Example of extension of the class Interpolation
###

        #--- class definition
        
        setClass("MMLoess",
# The new class extends Interpolation
                contains="Interpolation",
# and then defines its own attributes
                representation(span = "numeric",
                                                        degree = "integer",
                                                        model = "ANY"
                                                        ),
                prototype(      span=0.1,
                                                degree=as.integer(2),
                                                model = NULL
                                                )
        )

        #--- constructors

        setGeneric("MMLoess",
                function(val) standardGeneric("MMLoess"))
        
        setMethod("MMLoess",    
                signature(val="missing"),
                function(val){
                        object <- new("MMLoess")
                        object@visible <- TRUE
                        object@color <- "#000000"
                        object
                }
        )

        #--- accessors

        setGeneric("span", 
                function(object) standardGeneric("span"))
        
        setMethod("span","MMLoess",
                function(object)object@span)

        setGeneric("degree", 
                function(object) standardGeneric("degree"))
        
        setMethod("degree","MMLoess",
                function(object)object@degree)

        #--- replacement methods

        setGeneric("span<-",
                function(object,value) standardGeneric("span<-"))

        setReplaceMethod("span","MMLoess",
                function(object,value){
                        if(value > 0)
                                object@span <- value
                        object
                }
        )

        setGeneric("degree<-",
                function(object,value) standardGeneric("degree<-"))

        setReplaceMethod("degree","MMLoess",
                function(object,value){
                        if(value > 0 && value < 3)
                                object@degree <- as.integer(value)
                        object
                }
        )
        
# Here are the functions that must be redefined:

# query is the function that returns the recombination rate on a specific 
# physical point
        setMethod("query",c("MMLoess","numeric"),function(object,pos){
                posplus <- pos+1
                posmoins <- pos-1
                pred <- predict(object@model,newdata=c(posmoins,posplus))               
                round((pred[[2]]-pred[[1]])/((posplus-posmoins)/1000000),digits=2)
        })

# Interpolate is the function that carries out the calculation of the 
# interpolation. It must at least populate the vector rates with the 
# recombination rates at the position of each markers but it may also save 
# more detailed results elsewhere in the class.
        setMethod("interpolate",
                signature(object="MMLoess",map="MareyMap"),
                function(object,map){
                        valgen <- geneticDistances(map)[which(markerValidity(map))]
                        valphys <- physicalPositions(map)[which(markerValidity(map))]
                        object@physicalPositions <- valphys
                        object@model <- loess(  valgen ~ valphys, 
                                                                                        span=object@span,
                                                                                        degree=object@degree)
                        #physical positions +1
                        pp1 <- physicalPositions(map)+1
                        #physical positions -1
                        pm1 <- physicalPositions(map)-1
                        #predicted genetical positions @ phys+1
                        gp1 <- predict(object@model,newdata=pp1)
                        #predicted genetical positions @ phys-1
                        gm1 <- predict(object@model,newdata=pm1)
                        object@rates <- mapply(function(xa,xb,ya,yb){
                                round((yb-ya)/((xb-xa)/1000000),2)
                        },pm1,pp1,gm1,gp1)
                        object
                }
        )

# PlotModel is the function called to plot the interpolation's model of the 
# marey map. If this function is not redefined, nothing will be plotted 
        #--- plotModel
        setMethod("plotModel","MMLoess",function(object,...){
                cl <- match.call()
                cl[[1]] <- as.name("plot")
                cl$col <- object@color
                cl$type = "l"
                expli <- seq(from=min(object@physicalPositions),
                                                        to=max(object@physicalPositions),
                                                        length.out=1000)
                cl$x <- expli/1000000
                cl$y <- predict(object@model,newdata=data.frame(expli))
                eval(cl)
        })
        
# PlotModel is the function called to plot the recombination rates estimated 
# with this method for the current map.         
        setMethod("plotRate","MMLoess",function(object,...){
                cl <- match.call()
                cl[[1]] <- as.name("plot")
                cl$object <- NULL
                cl$col <- object@color
                cl$type <- "l"
                expli <- seq(from=min(object@physicalPositions),
                                                        to=max(object@physicalPositions),
                                                        length.out=1000)
                cl$x <- expli/1000000
                #physical positions +1
                pp1 <- expli+1
                #physical positions -1
                pm1 <- expli-1
                #predicted genetical positions @ phys+1
                gp1 <- predict(object@model,newdata=pp1)
                #predicted genetical positions @ phys-1
                gm1 <- predict(object@model,newdata=pm1)
                cl$y <- mapply(function(xa,xb,ya,yb){
                        (yb-ya)/((xb-xa)/1000000)
                },pm1,pp1,gm1,gp1)
                eval(cl)
        })
        
# createOrder is used when a map is saved to text file with its interpolations. 
# This function returns the order to recreate the interpolation when the file 
# read in. This will be saved at the top of the text file as a comment. 
        setMethod("createOrder",
                "MMLoess",
                function(object){
                        paste("new(\"MMLoess\",",argList(object),", span = ",object@span,
                                        ", degree = as.integer(",object@degree,"))",sep="")
                }
        )
        
# userParam returns information about the parameters accessible to the user 
# as a vector of objects of type interpolationParam. 
# These informations can be used by the GUI to dynamically create the dialog 
# to modify these parameters.
        setMethod("userParam",
                "MMLoess",
                function(object){       
                        span <- InterpolationParam()
                        paramName(span) <- "span"
                        paramDesc(span) <- "span\n"
                        paramType(span) <- "numeric"
                        paramDefault(span) <- 0.1
                        paramMin(span) <- 0
                        paramMax(span) <- 1     
                        paramFun(span) <- "span"
                        
                        deg <- InterpolationParam()
                        paramName(deg) <- "degree"
                        paramDesc(deg) <- "degree of the fitted polynomial"
                        paramType(deg) <- "integer"
                        paramDefault(deg) <- 2
                        paramMin(deg) <- 0
                        paramMax(deg) <- 2      
                        paramFun(deg) <- "degree"
                        # basic user params are defined in the class Interpolation.  
                        c(userParam(as(object,"Interpolation")),list(span,deg))
                }
        )

# this last line is necessary to register the method so that it is 
# accessible via the GUI
        registerInterpolationMethod("loess based method","MMLoess")



